/*
	gl_poly.c

	Poly list renderer for OpenGL

	Copyright (C) 2000       Joseph Carter <knghtbrd@debian.org>
	Copyright (C) 2000       Forest Hale <havoc@gamevisions.com>

	Author: Joseph Carter <knghtbrd@debian.org>
	Date: 15 June 2000

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to:

		Free Software Foundation, Inc.
		59 Temple Place - Suite 330
		Boston, MA  02111-1307, USA

	$Id$
*/

#ifdef HAVE_CONFIG_H
#	include "config.h"
#endif
#include <stdlib.h>

#include "glquake.h"
#include "gl_poly.h"
#include "console.h"

// ===========
// DEPTH POLYS
// ===========

depthvert_t *depthvert;
depthpoly_t *depthpoly;
unsigned short currentdepthpoly;
unsigned short currentdepthvert;


/*
	Poly_ClearDepthPolys
*/
void
Poly_ClearDepthPolys (void)
{
	currentdepthpoly = currentdepthvert = 0;
}


/*
	Poly_BeginDepthPoly
*/
void
Poly_BeginDepthPoly (void)
{
	if (currentdepthpoly >= MAX_DEPTHPOLYS
			|| currentdepthvert >= MAX_DEPTHVERTS)
		return;

	depthpoly[currentdepthpoly].firstvert = currentdepthvert;
	depthpoly[currentdepthpoly].verts = 0;	
}


/*
	Poly_EndDepthPoly
*/
void
Poly_EndDepthPoly (void)
{
	if (currentdepthpoly >= MAX_DEPTHPOLYS)
		return;

	if (depthpoly[currentdepthpoly].verts < 3)
	{
		// invalid poly - reset vert pointer
		currentdepthvert = depthpoly[currentdepthpoly].firstvert;
		return;
	}

	if (currentdepthvert >= MAX_DEPTHVERTS)
		return;
	currentdepthpoly++;		
}

/*
	Poly_DepthPolyVert
*/
void
Poly_DepthPolyVert (vec3_t v)
{
	if (currentdepthpoly >= MAX_DEPTHPOLYS
			|| currentdepthvert >= MAX_DEPTHVERTS)
		return;

	VectorCopy (depthvert[currentdepthvert].v, v);
	currentdepthvert++;
	depthpoly[currentdepthpoly].verts++;
}

/*
	Poly_RenderDepthPolys

	When drawn with a 0 colormask, a poly is transparent
*/
void
Poly_RenderDepthPolys (void)
{
	int		i, j;
	depthpoly_t	*p;
	depthvert_t	*vert;

	if (currentdepthpoly < 1)
		return;

	glColorMask (0, 0, 0, 0);
	glDepthMask (1);

	for (i = 0, p = depthpoly; i < currentdepthpoly; i++, p++)
	{
		vert = &depthvert[p->firstvert];
		glBegin (GL_TRIANGLE_FAN);
		for (j=0 ; j<p->verts ; j++, vert++)
			glVertex3fv (vert->v);
		glEnd ();
	}

	glColorMask (1, 1, 1, 1);	
}

// =================
// Poly housekeeping
// =================

/*
	Poly_Init
*/
void
Poly_Init (void)
{
	// depth polys
	depthvert = malloc(MAX_DEPTHVERTS * sizeof(depthvert_t));
	depthpoly = malloc(MAX_DEPTHPOLYS * sizeof(depthpoly_t));
}


/*
	Poly_Shutdown
*/
void
Poly_Shutdown (void)
{
	// depth polys
	free (depthvert);
	free (depthpoly);
}

